@@ -8,6 +8,33 @@ use constant POSCOL=>"blue";
 
 our $VERSION = '1.0';
 
+sub my_options {
+    {   
+        min_score => [
+            'integer',
+            undef,
+            "Minimum value of the signal graph feature's \"score\" attribute."],
+        max_score => [
+            'integer',
+            undef,
+            "Maximum value of the signal graph feature's \"score\" attribute."]
+    };
+}
+
+sub my_description {
+    return <<END;
+This glyph draws signal graph (wiggle_xyplot) using wiggle or BigWig files
+requires a special load gff file that uses attributes 'wigfileA' and 'wigfileB'
+For BigWig support it is also required to have 'fasta' attribute set to point
+to a fasta file for your organism of interest
+
+Example:
+
+2L  test   hybrid    5407   23011573    .     .     .     Name=Experiment 1;wigfileA=SomeWigFile1.wigdb;wigfileB=SomeWigFile2.wigdb
+
+END
+}
+
 #Checking the method for individual features (RNA-Seq reads)
 sub _check_uni {
  return shift->option('u_method') || 'match';
@@ -48,15 +75,31 @@ sub draw {
 
  #Draw dual graph if we don't have a score
  my @wiggles = ($feature->attributes('wigfileA'),$feature->attributes('wigfileB'));
+ my ($fasta)   = $feature->attributes('fasta');
  my($scale,$y_origin,$min_score,$max_score);
 
  $self->panel->startGroup($gd);
 
+ #Depending on what we have (wiggle or BigWig) pick the way to paint the signal graph
  for(my $w = 0; $w < @wiggles; $w++){
   if($w > 0){$self->configure(-pos_color, NEGCOL);}
   else{$self->configure(-pos_color, POSCOL);}
-  
-  $self->draw_wigfile($feature,$wiggles[$w],@_) if $wiggles[$w];
+  if ($wiggles[$w] =~ /\.wi\w{1,3}$/) {
+   $self->draw_wigfile($feature,$wiggles[$w],@_);
+  } elsif ($wiggles[$w] =~ /\.bw$/ && $fasta) {
+   use Bio::DB::BigWig 'binMean';
+   use Bio::DB::Sam;
+   my $wig = Bio::DB::BigWig->new(-bigwig => "$wiggles[$w]",
+                                  -fasta  => Bio::DB::Sam::Fai->open("$fasta"));
+
+   my ($summary) = $wig->features(-seq_id => $feature->segment->ref,
+                                  -start  => $self->panel->start,
+                                  -end    => $self->panel->end,
+                                  -type   => 'summary');
+   my $stats = $summary->statistical_summary($self->width);
+   my @vals  = map {$_->{validCount} ? $_->{sumData}/$_->{validCount}:0} @$stats;
+   $self->draw_coverage($self,\@vals,@_);
+  }
  }
 }
 
